
library(mice)
missing.no <- function(data,n,perc){
  missing.data <- vector("list",length=n)
  m <- ncol(data)*nrow(data)
  set.seed(23456)   ## use this seed
  missing.data = lapply(1:n,function(x) sample(1:m,round(m*perc)))
  return(missing.data)
}

loc.gen.missing <- function(mv.no,Type,nr,nc){
  col <- floor(mv.no/nr)+1
  row <- mv.no-nr*floor(mv.no/nr)
  if(row==0){(row=nr) & (col=col-1)}
  miss.type <- Type[col]  # type of missing value
  return(c(row,col,miss.type))
}
determin.i <- function(k.imputed,weight,miss.type,w){
  ## weight could be adjusted according to user's preference
  if (miss.type%in%"con"){imputed.value <- weighted.mean(k.imputed,weight^w)}
  else if(miss.type%in%"nom"){ 
    p.number <- length(table(k.imputed))
    weight.sum <- tapply(weight,k.imputed,sum)
    if (length(max(weight.sum))>1){imputed.value <- sample(as.numeric(names(which.max(weight.sum))),1)}
    else{imputed.value <- as.numeric(names(which.max(weight.sum)))}
  } 
  else {imputed.value <- round(weighted.mean(k.imputed,weight^w))}
  #if(length(imputed.value)>1){imputed.value <- sample(imputed.value,1)}
  return(imputed.value)  
}
Evaluate <- function(imputed.value,true.value,miss.type,col,Complete){
  if (miss.type=="con"){
    ## normalized square error
    ## could be >1
    accu.v <-((true.value-imputed.value)/sd(na.omit(Complete[,col])))^2
  }
  else if (miss.type=="nom"){
    ## 0~1
    if(true.value==imputed.value){accu.v <- 0}
    else {accu.v <- 1}
  }
  else {
    ## 0~1
    accu.v <- ((true.value-imputed.value)/(nlevels(as.factor(Complete[,col]))-1))^2}
  accu.v  
}

Summary.Result <- function(Error,miss.type){
  result.con <- sqrt(mean(Error[miss.type=="con"]))
  result.nom <- table(Error[miss.type=="nom"])[row.names(as.matrix(table(Error[miss.type=="nom"])))==1]/sum(table(Error[miss.type=="nom"]))
  result.ord <- sqrt(mean(Error[miss.type=="ord"]))  
  return(c(result.con,result.nom,result.ord))
}
MICE.function <- function(ind){
  nc <- ncol(Complete)
  nr <- nrow(Complete)
  Missing <- round(perc*nc*nr) 
  mv.comp <- Complete
  mv.no = missing.data[[ind]]
  ## position of missing
  missing.info = t(sapply(mv.no,function(x) loc.gen.missing(x,Type,nr,nc)))
  col <- as.numeric(missing.info[,2])
  row <- as.numeric(missing.info[,1])
  miss.type <- missing.info[,3]  
  ## generate missingness   
  for (j in 1:Missing)
  {
    mv.comp[row[j],col[j]] <- NA
  }
  mv.comp.2 <- as.data.frame(mv.comp)
  for(i in 1:length(which(Type=="con"))){
    mv.comp.2[,which(Type=="con")[i]] <- as.numeric(mv.comp[,which(Type=="con")[i]])}
  if(sum(Type=="ord")>0){
    for(i in 1:length(which(Type=="ord"))){
      mv.comp.2[,which(Type=="ord")[i]] <- as.ordered(mv.comp[,which(Type=="ord")[i]])}
  }
  if(sum(Type=="nom")>0){
      for(i in 1:length(which(Type=="nom"))){
        mv.comp.2[,which(Type=="nom")[i]] <- as.factor(mv.comp[,which(Type=="nom")[i]])} 
  }

  imp <- mice(mv.comp.2,m=5,pred=quickpred(mv.comp.2, minpuc=0.25),defaultMethod = c("pmm", "cart", "cart", "pmm"),seed=12345)
  Imputed.data = lapply(1:5,function(x) complete(imp,x))
  Imputed.value <- sapply(1:Missing,function(x) sapply(1:5,function(y) Imputed.data[[y]][row[x],col[x]]))


  Imputed.value.final = sapply(1:Missing,function(x) determin.i(Imputed.value[,x],rep(1,5),Type[col[x]],1))
  if(length(na.omit(Imputed.value.final))<Missing){temp = which(is.na(Imputed.value.final))
                                                  Imputed.value.final = Imputed.value.final[-temp]
                                                  row = row[-temp]
                                                  col = col[-temp]
							 Missing=length(unlist(Imputed.value.final))
  }
  else{Imputed.value.final=unlist(Imputed.value.final)}

  accu.mice <- sapply(1:Missing,function(x) Evaluate(Imputed.value.final[x],Complete[row[x],col[x]],Type[col[x]],col[x],Complete))
  result = Summary.Result(accu.mice,Type[col])
  names(result) = c("con","nom","ord")[which(c("con","nom","ord")%in%Type)]
  return(list(result,accu.mice,row,col))  
}
