## Random forest
library(missForest)
missing.no <- function(data,n,perc){
  missing.data <- vector("list",length=n)
  m <- ncol(data)*nrow(data)
  set.seed(23456)   ## use this seed
  missing.data = lapply(1:n,function(x) sample(1:m,round(m*perc)))
  return(missing.data)
}
loc.gen.missing <- function(mv.no,Type,nr,nc){
  col <- floor(mv.no/nr)+1
  row <- mv.no-nr*floor(mv.no/nr)
  if(row==0){(row=nr) & (col=col-1)}
  miss.type <- Type[col]  # type of missing value
  return(c(row,col,miss.type))
}

Evaluate <- function(imputed.value,true.value,miss.type,col,Complete){
  if (miss.type=="con"){
    ## normalized square error
    ## could be >1
    accu.v <-((true.value-imputed.value)/sd(na.omit(Complete[,col])))^2
  }
  else if (miss.type=="nom"){
    ## 0~1
    if(true.value==imputed.value){accu.v <- 0}
    else {accu.v <- 1}
  }
  else {
    ## 0~1
    min.value = min(Complete[,col])
    max.value = max(Complete[,col])
    accu.v <- ((true.value-max(min.value,min(round(imputed.value),max.value)))/(nlevels(as.factor(Complete[,col]))-1))^2}
  accu.v  
}

Summary.Result <- function(Error,miss.type){
  result.con <- sqrt(mean(Error[miss.type=="con"]))
  result.nom <- table(Error[miss.type=="nom"])[row.names(as.matrix(table(Error[miss.type=="nom"])))==1]/sum(table(Error[miss.type=="nom"]))
  result.ord <- sqrt(mean(Error[miss.type=="ord"]))  
  return(c(result.con,result.nom,result.ord))
}


missForest <- function (xmis, maxiter = 10, ntree = 100, variablewise = FALSE, VarType,
                        decreasing = FALSE, verbose = FALSE, mtry = floor(sqrt(ncol(xmis))), 
                        replace = TRUE, classwt = NULL, cutoff = NULL, strata = NULL, 
                        sampsize = NULL, nodesize = NULL, maxnodes = NULL, xtrue = NA) 
{
  n <- nrow(xmis)
  p <- ncol(xmis)
  if (!is.null(classwt)) 
    stopifnot(length(classwt) == p, typeof(classwt) == "list")
  if (!is.null(cutoff)) 
    stopifnot(length(cutoff) == p, typeof(cutoff) == "list")
  if (!is.null(strata)) 
    stopifnot(length(strata) == p, typeof(strata) == "list")
  if (!is.null(nodesize)) 
    stopifnot(length(nodesize) == 2)
  if (any(apply(is.na(xmis), 2, sum) == n)) {
    indCmis <- which(apply(is.na(xmis), 2, sum) == n)
    xmis <- xmis[, -indCmis]
    p <- ncol(xmis)
    cat("  removed variable(s)", indCmis, "due to the missingness of all entries\n")
  }
  ximp <- xmis
  xAttrib <- lapply(xmis, attributes)
  varType <- character(p)
  for (t.co in 1:p) {
    if (is.null(xAttrib[[t.co]])) {
      varType[t.co] <- "numeric"
      ximp[is.na(xmis[, t.co]), t.co] <- mean(xmis[, t.co], 
                                              na.rm = TRUE)
    }
    else {
      varType[t.co] <- "factor"
      max.level <- max(table(ximp[, t.co]))
      class.assign <- sample(names(which(max.level == summary(ximp[, 
                                                                   t.co]))), 1)
      if (class.assign != "NA's") {
        ximp[is.na(xmis[, t.co]), t.co] <- class.assign
      }
      else {
        while (class.assign == "NA's") {
          class.assign <- sample(names(which(max.level == 
                                               summary(ximp[, t.co]))), 1)
        }
        ximp[is.na(xmis[, t.co]), t.co] <- class.assign
      }
    }
  }
  varType <- VarType
  NAloc <- is.na(xmis)
  noNAvar <- apply(NAloc, 2, sum)
  sort.j <- order(noNAvar)
  if (decreasing) 
    sort.j <- rev(sort.j)
  sort.noNAvar <- noNAvar[sort.j]
  Ximp <- vector("list", maxiter)
  iter <- 0
  k <- length(unique(varType))
  convNew <- rep(0, k)
  convOld <- rep(Inf, k)
  OOBerror <- numeric(p)
  names(OOBerror) <- varType
  if (k == 1) {
    if (unique(varType) == "numeric") {
      names(convNew) <- c("numeric")
    }
    else {
      names(convNew) <- c("factor")
    }
    convergence <- c()
    OOBerr <- numeric(1)
  }
  else {
    names(convNew) <- c("numeric", "factor")
    convergence <- matrix(NA, ncol = 2)
    OOBerr <- numeric(2)
  }
  stopCriterion <- function(varType, convNew, convOld, iter, 
                            maxiter) {
    k <- length(unique(varType))
    if (k == 1) {
      (convNew < convOld) & (iter < maxiter)
    }
    else {
      ((convNew[1] < convOld[1]) | (convNew[2] < convOld[2])) & 
        (iter < maxiter)
    }
  }
  while (stopCriterion(varType, convNew, convOld, iter, maxiter)) {
    if (iter != 0) {
      convOld <- convNew
      OOBerrOld <- OOBerr
    }
    cat("  missForest iteration", iter + 1, "in progress...")
    t.start <- proc.time()
    ximp.old <- ximp
    for (s in 1:p) {
      varInd <- sort.j[s]
      if (noNAvar[[varInd]] != 0) {
        obsi <- !NAloc[, varInd]
        misi <- NAloc[, varInd]
        obsY <- ximp[obsi, varInd]
        obsX <- ximp[obsi, seq(1, p)[-varInd]]
        misX <- ximp[misi, seq(1, p)[-varInd]]
        typeY <- varType[varInd]
        if (typeY == "numeric") {
          RF <- randomForest(x = obsX, y = obsY, ntree = ntree, 
                             mtry = mtry, replace = replace, sampsize = if (!is.null(sampsize)) 
                               sampsize[[varInd]]
                             else if (replace) 
                               nrow(obsX)
                             else ceiling(0.632 * nrow(obsX)), nodesize = if (!is.null(nodesize)) 
                               nodesize[1]
                             else 1, maxnodes = if (!is.null(maxnodes)) 
                               maxnodes
                             else NULL)
          OOBerror[varInd] <- RF$mse[ntree]
          misY <- predict(RF, misX)
        }
        else {
          obsY <- factor(obsY)
          summarY <- summary(obsY)
          if (length(summarY) == 1) {
            misY <- factor(rep(names(summarY), sum(misi)))
          }
          else {
            RF <- randomForest(x = obsX, y = obsY, ntree = ntree, 
                               mtry = mtry, replace = replace, classwt = if (!is.null(classwt)) 
                                 classwt[[varInd]]
                               else rep(1, nlevels(obsY)), cutoff = if (!is.null(cutoff)) 
                                 cutoff[[varInd]]
                               else rep(1/nlevels(obsY), nlevels(obsY)), 
                               strata = if (!is.null(strata)) 
                                 strata[[varInd]]
                               else obsY, sampsize = if (!is.null(sampsize)) 
                                 sampsize[[varInd]]
                               else if (replace) 
                                 nrow(obsX)
                               else ceiling(0.632 * nrow(obsX)), nodesize = if (!is.null(nodesize)) 
                                 nodesize[2]
                               else 5, maxnodes = if (!is.null(maxnodes)) 
                                 maxnodes
                               else NULL)
            OOBerror[varInd] <- RF$err.rate[[ntree, 1]]
            misY <- predict(RF, misX)
          }
        }
        ximp[misi, varInd] <- misY
      }
    }
    cat("done!\n")
    iter <- iter + 1
    Ximp[[iter]] <- ximp
    t.co2 <- 1
    for (t.type in names(convNew)) {
      t.ind <- which(varType == t.type)
      if (t.type == "numeric") {
        convNew[t.co2] <- sum((ximp[, t.ind] - ximp.old[, 
                                                        t.ind])^2)/sum(ximp[, t.ind]^2)
      }
      else {
        dist <- sum(as.character(as.matrix(ximp[, t.ind])) != 
                      as.character(as.matrix(ximp.old[, t.ind])))
        convNew[t.co2] <- dist/(n * sum(varType == "factor"))
      }
      t.co2 <- t.co2 + 1
    }
    if (!variablewise) {
      NRMSE <- sqrt(mean(OOBerror[varType == "numeric"])/var(as.vector(as.matrix(xmis[, 
                                                                                      varType == "numeric"])), na.rm = TRUE))
      PFC <- mean(OOBerror[varType == "factor"])
      if (k == 1) {
        if (unique(varType) == "numeric") {
          OOBerr <- NRMSE
          names(OOBerr) <- "NRMSE"
        }
        else {
          OOBerr <- PFC
          names(OOBerr) <- "PFC"
        }
      }
      else {
        OOBerr <- c(NRMSE, PFC)
        names(OOBerr) <- c("NRMSE", "PFC")
      }
    }
    else {
      OOBerr <- OOBerror
      names(OOBerr)[varType == "numeric"] <- "MSE"
      names(OOBerr)[varType == "factor"] <- "PFC"
    }
    if (any(!is.na(xtrue))) {
      err <- suppressWarnings(mixError(ximp, xmis, xtrue))
    }
    if (verbose) {
      delta.start <- proc.time() - t.start
      if (any(!is.na(xtrue))) {
        cat("    error(s):", err, "\n")
      }
      cat("    estimated error(s):", OOBerr, "\n")
      cat("    difference(s):", convNew, "\n")
      cat("    time:", delta.start[3], "seconds\n\n")
    }
  }
  if (iter == maxiter) {
    if (any(is.na(xtrue))) {
      out <- list(ximp = Ximp[[iter]], OOBerror = OOBerr)
    }
    else {
      out <- list(ximp = Ximp[[iter]], OOBerror = OOBerr, 
                  error = err)
    }
  }
  else {
    if (any(is.na(xtrue))) {
      out <- list(ximp = Ximp[[iter - 1]], OOBerror = OOBerrOld)
    }
    else {
      out <- list(ximp = Ximp[[iter - 1]], OOBerror = OOBerrOld, 
                  error = suppressWarnings(mixError(Ximp[[iter - 
                                                            1]], xmis, xtrue)))
    }
  }
  class(out) <- "missForest"
  return(out)
}

RandomForest.function <- function(ind){
  nc <- ncol(Complete)
  nr <- nrow(Complete)
  Missing <- round(perc*nc*nr) 
  mv.comp <- Complete
  mv.no = missing.data[[ind]]
  ## position of missing
  missing.info = t(sapply(mv.no,function(x) loc.gen.missing(x,Type,nr,nc)))
  col <- as.numeric(missing.info[,2])
  row <- as.numeric(missing.info[,1])
  miss.type <- missing.info[,3]  
  ## generate missingness   
  for (j in 1:Missing)
  {
    mv.comp[row[j],col[j]] <- NA
  }
  Type.input = Type
  Type.input[which(Type%in%c("con","ord"))] = "numeric"
  Type.input[which(Type%in%c("nom"))] = "factor"
  Rf.imputed.data <- missForest(mv.comp, VarType=Type.input,maxiter=3,  verbose = TRUE)
  print("random forest")
  accu.rf <- sapply(1:Missing,function(x) Evaluate(Rf.imputed.data[[1]][row[x],col[x]],Complete[row[x],col[x]],miss.type[x],col[x],Complete))
  print("random forest finish")
  result = Summary.Result(accu.rf,miss.type)
  names(result) = c("con","nom","ord")[which(c("con","nom","ord")%in%Type)]
  return(list(result,accu.rf)) 
}

